#!/bin/sh

APPNAME=vixn

cd "`dirname "$0"`"
srcdir="`pwd`"

if [ -n "$1" ] ; then
    mkdir -p "$1"
    cd "$1"
    prefix="`pwd`"
else
    prefix=/usr/local
fi

libdir="$prefix/lib/$APPNAME"
datadir="$prefix/share"
confdir="$datadir/$APPNAME/config"
docdir="$prefix/share/doc/$APPNAME/vixn"
bindir="$prefix/bin"
icondir="$datadir/icons/hicolor/scalable/apps"
desktopdir="$datadir/applications"
dbusdir="$datadir/dbus-1/services"

cd "$srcdir"

echo "Creating/ensuring directories:"
echo $libdir $confdir $bindir
echo $icondir $desktopdir $dbusdir
echo $docdir
install -d "$libdir/vixn" "$confdir" "$docdir/html" "$bindir" \
        "$icondir" "$desktopdir" "$dbusdir"

echo "Installing python code"
install vixn.py vixgrep "$libdir"
install --mode=0644 vixn/*.py "$libdir/vixn"
echo "Compiling python code"
cd "$libdir/vixn"
python3 -O - <<SCRIPT
import compileall
compileall.compile_dir('.')
SCRIPT
cd "$srcdir"

relative()
{
    # Prints path $1 converted from absolute to relative to $2
    python3 -c "
import os
p1 = '$1'.split(os.path.sep)
p2 = '$2'.split(os.path.sep)
if os.path.isdir('$2'):
    p2.append(p1[-1])
for n in range(min(len(p1), len(p2))):
    if p1[n] != p2[n]:
        break
p3 = ['..'] * (len(p2) - n - 1) + p1[n:]
print(os.path.sep.join(p3))
"
}

echo "Linking executables in $bindir"
#ln -nfs "$libdir/vixn.py" "$bindir/vixn"
#ln -nfs "$libdir/vixgrep" "$bindir/vixgrep"
ln -nfs `relative "$libdir/vixn.py" "$bindir/vixn"` "$bindir/vixn"
ln -nfs `relative "$libdir/vixgrep" "$bindir/vixgrep"` "$bindir/vixgrep"

echo "Installing data"
install --mode=0755 -d "$confdir/languages"
install --mode=0644 -t "$confdir/languages" config/languages/*
install --mode=0644 config/menus "$confdir"
install --mode=0644 vixn_logo.svg "$datadir/$APPNAME"
install --mode=0644 vixn_logo.svg "$icondir/vixn.svg"
install --mode=0644 vixn.desktop "$desktopdir"
if [ "$prefix" = "/usr/local" ] ; then
    install --mode=0644 org.vixn.service "$dbusdir"
else
    sed "s|/usr/local|${prefix}|" < org.vixn.service \
            > "$dbusdir/org.vixn.service"
fi


if [ ! -f ChangeLog ] ; then
    echo "Creating ChangeLog with git"
    set +e
    git log > ChangeLog && ln -s ../ChangeLog docs/ChangeLog
    set -e
fi
echo "Installing docs"
[ -f ChangeLog ] && install --mode=0644 ChangeLog "$docdir"
install --mode=0644 docs/COPYING docs/README "$docdir"
install --mode=0644 docs/html/*.html docs/html/*.txt "$docdir/html"
install --mode=0755 docs/html/list_filetypes "$docdir/html"

echo '**** Did you remember to run makedocs.py first? *****'
