#!/bin/sh

cd `dirname "$0"`

if [ "$1" = "-f" ] ; then
    FORCE=-f
    shift
fi

if [ -n "$1" ] ; then
    version="$1"
else
    version=`grep '<Version>' AppInfo.xml | sed 's/^[^>]*>//;s/ .*//'`
    echo "No version specified, read $version from AppInfo.xml"
fi

if [ -n "$1" ] && echo $version | grep -qv -- -g ; then
    echo "Creating git tag"
    git tag $FORCE -s -m "Release $version" $version
fi
./stamp-version

echo "git ChangeLog"
git log > ChangeLog

echo "Building docs"
./makedocs.py

tarfile=vixn-${version}.tar.bz2
echo "Building $tarfile"
# Need to touch tarball to make sure something of that name already exists
# otherwise tar complains about directory contents changing
touch "${tarfile}"
tar -c --bzip2 -f "${tarfile}" \
        --exclude='.git*' --exclude='*.pyc' --exclude='*.pyo' \
        --exclude='vixn-*.tar*' --transform "s/^\./vixn-${version}/" .
