#!/usr/bin/env python3
""" Start-up. """

import optparse
import os
import sys
import time
import vixn.version
import vixn.vixi18n

#import gi

prog = 'vixn'

def start_vixn(replace, started_by_dbus, fgparent, args, always_reload):
    from gi.repository import Gtk
    from vixn import dialog, main, msgs, vixdbus
    run_direct = True
    cwd = os.getcwd()
    try:
        if vixdbus.dbus_ok or vixdbus.init_dbus():
            if not vixdbus.start_main_service(replace):
                if vixdbus.run_vixn_via_dbus(cwd, fgparent, args,
                        always_reload):
                    run_direct = False
    except:
        msgs.exception(_("Failed to initialise D-BUS"))
    if run_direct:
        sys.excepthook = dialog.excepthook
        main.run(cwd, started_by_dbus, fgparent, args, always_reload)
        Gtk.main()

def parse_args(argv):
    """ Returns [opts, argv] where opts has attributes corresponding to
    each of thes options and argv contains the remaining arguments. """
    parser = optparse.OptionParser(version = vixn.version.version_string,
            description = _("A modern vi-like text editor."),
            prog = prog)
    parser.add_option('-f', '--foreground', '--nofork', action = 'store_true',
            help = _("Run in foreground"))
    parser.add_option('-r', '--reload', action = 'store_true',
            help = _("Load a fresh copy of each file even if already in " \
                    "memory (not recommended: likely to confuse throwbacks)"))
    parser.add_option('--replace', action = 'store_true',
            help = _("Replace an existing instance of vixn as owner of " \
                    "vixn's D-BUS service. Ignored if -f is set."))
    parser.add_option('--dbus-service', action = 'store_true',
            help = _("Indicates that vixn is being started by a dbus " \
                    "service file (don't fork and don't open an empty window)"))
    return parser.parse_args(argv)

def init():
    replace = False
    opts, args = parse_args(sys.argv)
    if opts.dbus_service:
        start_vixn(True, True, None, args, opts.reload)
        return 0
    # If --nofork use pipe to make sure child doesn't open buffers until
    # waiter is ready
    if opts.foreground:
        fgparent = "%d_%d" % (os.getpid(), int(time.time()))
        [r, w] = os.pipe()
    else:
        fgparent = None
    if not os.fork():
        # Child of fork
        if opts.foreground:
            ok = os.read(r, 2)
        start_vixn(opts.replace, False, fgparent, args, opts.reload)
        return 0
    elif opts.foreground:
        from gi.repository import Gtk
        from vixn import vixdbus
        # Parent of fork; waiter waits for all spawned buffers to close
        waiter = vixdbus.Waiter(fgparent)
        # Tell child when ready to receive buffer messages
        os.write(w, b"OK")
        Gtk.main()
    return 0

if __name__ == '__main__':
    sys.exit(init())
