import os.path
import re

from . import buffers
from . import settings
from . import throwback
from . import vixwin


_grep_buffer_index = 0

sessions = {}


class GrepBufExtn(throwback.BufExtn):
    def __init__(self, cwd):
        global _grep_buffer_index, sessions
        tbfilename, _grep_buffer_index = \
                buffers.generate_filename(cwd,
                        _("grep_results_%d"), _grep_buffer_index)
        self.session_id = _grep_buffer_index
        sessions[_grep_buffer_index] = self
        from . import vixbuf
        tbbuf = vixbuf.VixnBuffer(settings.defaults, cwd, None, tbfilename)
        throwback.BufExtn.__init__(self, tbbuf)
        vixwin.VixnWindow(tbbuf, settings.PRIORITY_DEFAULT).present()
        throwback.add_view_extn_to_buf(tbbuf)

    def parse_line(self, line, *args, **kwargs):
        try:
            f, l, t = line.split(':', 2)
            if not os.path.isabs(f):
                f = os.path.join(self.buf.get_cwd(), f)
            start = line.index(':') + 1
            return [f, int(l), 0, start, line.index(':', start),
                    None, None, "find"]
        except:
            return None

    def destroy(self):
        throwback.BufExtn.destroy(self)
        global sessions
        del sessions[self.session_id]

    def finished(self):
        self.buf.set_modified(False)


def open_session(cwd):
    return GrepBufExtn(cwd).session_id


settings.register("grep", description = _("Pathname of grep executable"))

