""" Manage GTKSourceView languages. """

from gi.repository import Gio, GtkSource

from . import dirs

_manager = None


def get_manager():
    global _manager
    if not _manager:
        _manager = GtkSource.LanguageManager()
        ldirs = dirs.get_all_config("languages")
        if ldirs:
            # Reverse because we're prepending but want to preserve order
            # returned by dirs
            ldirs.reverse()
            paths = _manager.get_search_path()
            for d in ldirs:
                paths = [d] + paths
            _manager.set_search_path(paths)
    return _manager


def compression_test(filename):
    """ If a filename ends in .bz2 or .gz returns [basename, extension];
    extension is converted to lower case. Returns [filename, None] if not
    bz2 or gz. """
    base_ext = filename.rsplit('.', 1)
    ext = None
    if len(base_ext) == 2:
        ext = base_ext[1].lower()
        if ext == 'gz' or ext == 'bz2':
            filename = base_ext[0]
    return [filename, ext]


def guess_filetype(filename, text):
    if filename[-1] == '~':
        filename = filename[:-1]
    filename = compression_test(filename)[0]
    ctype, certain = Gio.content_type_guess(filename, bytes(text, "utf8"))
    lang = get_manager().guess_language(filename, ctype)
    if lang:
        return lang.get_id()
    else:
        return None


def get_lang_by_name(name):
    return get_manager().get_language(name.lower())
