#!/usr/bin/env python
""" Basically glues everything else together. """

import optparse
import os
import sys

from gi.repository import Gtk

from . import dialog
from . import dirs
from . import openfile
from . import rc
from . import settings
from . import vixi18n
from . import vixexcept


class ArgError(vixexcept.UserError):
    def __init__(self, str = None):
        if not str:
            str = _("Line number argument given without file")
        vixexcept.UserError.__init__(self, str)


def parse_args(fgparent, args):
    """ Returns [files, commands] where commands is a list of commands to
    execute on each file when opened. files is an array of arrays:
    (filename, line_number). line_numbers start from 1. 0 is used if line not
    specified. """
    files = []
    commands = []
    l = len(args)
    i = 1
    while i < l:
        try:
            a = args[i]
            if a[0] == '+':
                if a[1:].isdigit():
                    if i == l - 1:
                        # +nn as final arg is OK as long as it's a file
                        if os.path.exists(a):
                            files.append([a, 0, fgparent])
                        else:
                            raise ArgError()
                    else:
                        n = int(a[1:])
                        i += 1
                        a = args[i]
                        if a[0] == '+' and a[1:].isdigit() and \
                                not os.path.exists(a):
                            raise ArgError()
                        files.append([a, n, fgparent])
                else:
                    commands.append(a[1:])
            else:
                files.append([a, 0, fgparent])
        except ArgError as e:
            dialog.error(None, str(e))
        i += 1
    return [files, commands]


def run(cwd, started_by_dbus, fgparent, argv, always_reload):
    """ argv includes program name ie as in sys.argv[0]. """
    stngs = settings.defaults
    files, commands = parse_args(fgparent, argv)
    if not len(files) and not started_by_dbus:
        openfile.open_file(stngs, cwd, fgparent = fgparent, commands = commands,
                always_reload = always_reload)
    else:
        for [f, l, fgp] in files:
            try:
                stngs = openfile.open_file(stngs, cwd, f, l, fgp, commands,
                        always_reload)
            except UnicodeError:
                pass
            except:
                dialog.exception(None, _("Unable to load '%s'") % f)

Gtk.Window.set_default_icon_from_file(dirs.get_vixn_logo_icon())

rc.context.load_menus()
rc.context.load_config()
rc.context.load_auto_save()
