""" Miscellaneous utility functions etc. """

import os

from gi.repository import Gtk

from . import msgs

def custom_stock_button(text, stock, use_underline = True):
    """ Creates a button with a stock icon but custom text. """
    button = Gtk.Button(text, stock)
    widget = button
    while not isinstance(widget, Gtk.HBox):
        try:
            if isinstance(widget, Gtk.Bin):
                widget = widget.get_child()
            else:
                break
            label = widget.get_children()[1]
            label.set_text_with_mnemonic(text)
            return button
        except:
            pass
    msgs.warning(_("Unable to change text in '%s' stock button to '%s'") %
            (stock, text))
    return button


def esc_xml_text(sin, attr = False):
    sout = ""
    for c in sin:
        if c == '<':
            sout += '&lt;'
        elif c == '>':
            sout += '&gt;'
        elif c == '&':
            sout += '&amp;'
        elif c == '"' and attr:
            sout += '&quot;'
        elif ord(c) < 32 or ord(c) == 127:
            if attr:
                sout += r'\x%02x' % ord(c)
            else:
                sout += '&#%d;' % ord(c)
        else:
            sout += c
    return sout

def esc_xml_attr(sin):
    return esc_xml_text(sin, True)

def xml_attr_to_str(name, value):
    return " " + name + '="' + esc_xml_attr(str(value)) + '"'


def find_exe_in_path(leafnames):
    """ Uses find_exe_in_path to find first of list of leafnames. leafnames may
     be a single string and each element may be a full pathname. """
    paths = os.environ.get('PATH')
    if not paths:
        paths = os.defpath
    paths = paths.split(os.pathsep)
    if hasattr(leafnames, 'isspace'):
        leafnames = [leafnames]
    for l in leafnames:
        if os.path.dirname(l):
            l = os.path.abspath(l)
            if os.access(l, os.X_OK):
                return l
        else:
            for p in paths:
                f = os.path.join(p, l)
                if os.access(f, os.X_OK):
                    return f
    return None

