"""Logging of informational and error messages. """

import logging
import sys

from . import buffers

MESSAGE = 25
logging.addLevelName(MESSAGE, "MESSAGE")

logging.getLogger().setLevel(logging.DEBUG)

recursive_exception = False

def fatal(message):
    """Fatal error, causes an exit."""
    logging.critical(message)
    sys.exit(1)

def error(message):
    """We can continue, but something needs fixing,
    probably in the program."""
    logging.error(message)

def warning(message):
    """This is a problem, usually not caused by a bug in boxstar's code,
    eg an external file or program is missing or failed,
    or a config file has a syntax error."""
    logging.warning(message)

def message(message):
    """Useful information for users."""
    logging.log(MESSAGE, message)

def info(message):
    """Useful information for developers."""
    logging.info(message)

def debug(message):
    """Additional information for developers."""
    return
    logging.debug(message)

def exception(message):
    logging.exception(message)

def fatal_exception(message):
    sys.stderr.write(_('FATAL '))
    exception(message)
    global recursive_exception
    if not recursive_exception:
        recursive_exception = True
        buffers.save_all(True)
    sys.exit(1)

