""" Keystroke recording (vi command 'q') """

from gi.repository import GLib, Gtk

from . import misc

recordings = {}


def key_event_to_xml(e):
    s = '<ks' + misc.xml_attr_to_str('state', int(e.state))
    s += misc.xml_attr_to_str('keyval', int(e.keyval))
    if hasattr(e, 'hardware_keycode'):
        s += misc.xml_attr_to_str('hardware_keycode', int(e.hardware_keycode))
    if e.group:
        s += misc.xml_attr_to_str('group', int(e.group))
    return s + '/>'



class Recording(object):
    def __init__(self, reg):
        """ reg is the "register" character, ie what was pressed after q when
        starting the recording """
        global recordings
        self.reg = reg
        recordings[reg] = self
        self.keystrokes = []

    def add_keystroke(self, event):
        self.keystrokes.append(event.copy())

    def playback(self, view):
        # Actual playback deferred to idle handler to make sure vicmd
        # interpreter has been reset to appropriate state
        GLib.idle_add(self.__playback_callback,
                (view, Gtk.get_current_event_time()))

    def __playback_callback(self, args):
        view = args[0]
        t = Gtk.get_current_event_time()
        if not t:
            t = args[1]
        buf = view.get_buffer()
        iua = buf.ignore_user_action
        buf.ignore_user_action = True
        buf.begin_user_action()
        for e in self.keystrokes:
            e.window = Gtk.Widget.get_window(view)
            e.time = int(t)
            view.emit("key_press_event", e)
        buf.end_user_action()
        buf.ignore_user_action = iua
        return False

    def to_xml(self):
        s = '<qrec' + misc.xml_attr_to_str('reg', self.reg) + '>\n'
        for k in self.keystrokes:
            s += key_event_to_xml(k) + '\n'
        return s + '</qrec>\n'
