from gi.repository import Gdk, Gtk

from . import dirs
from . import msgs

MAIN_PREFIX = "<Vixn-Main>"
MAIN_PATH = MAIN_PREFIX + '/'

path_from_method = {}
method_from_path = {}

def strip_mnemonic(label):
    """ Strips mnemonics from menu labels. """
    return label.replace('_', '')

def register_item(path, method):
    global path_from_method
    global method_from_path
    path_from_method[method] = path
    method_from_path[path] = method

def warn_load(filename, message):
    msgs.warning(_("Error loading shortcuts file '%s': %s") % \
            (filename, message))

_handler_id = None

def connect_handler():
    global _handler_id
    if _handler_id == None:
        _handler_id = Gtk.AccelMap.get().connect("changed", _accel_changed_cb)
        return True
    else:
        return False

def disconnect_handler():
    global _handler_id
    if _handler_id != None:
        Gtk.AccelMap.get().disconnect(_handler_id)
        _handler_id = None
        return True
    else:
        return False

def blank_all(keep_disconnected = False):
    was_connected = disconnect_handler()
    def blank_one(data, path, key, mods, changed):
        Gtk.AccelMap.change_entry(path, Gdk.KEY_VoidSymbol, 0, True)
    Gtk.AccelMap.foreach(None, blank_one)
    if was_connected:
        connect_handler()

def load_main_from_file(filename):
    disconnect_handler()
    blank_all(True)
    fp = open(filename, 'r')
    for l in fp.readlines():
        l.strip()
        sp = l.split('=', 1)
        if len(sp) != 2:
            warn_load(filename, _("Invalid line '%s'") % l)
            continue
        method, accel = sp
        accel = accel.rstrip('\n').strip()
        key, mods = Gtk.accelerator_parse(accel)
        if not key:
            warn_load(filename,
                    _("'%s' is not a valid accelerator name") % accel)
            continue
        path = path_from_method.get(method.strip())
        if not path:
            warn_load(filename, 
                    _("Method '%s' has no menu path to apply shortcut") % \
                            method)
            continue
        path = MAIN_PATH + path
        if Gtk.AccelMap.lookup_entry(path):
            Gtk.AccelMap.change_entry(path, key, mods, True)
        else:
            Gtk.AccelMap.add_entry(path, key, mods)
    fp.close()
    connect_handler()

def save_main_to_file(filename):
    def write_shortcut_line(path, key, mods, changed, fp):
        if not path.startswith(MAIN_PATH):
            return
        path = path.split('/', 1)[1]
        method = method_from_path.get(path)
        if not method:
            msgs.error(_("No method for menu path '%s'") % path)
            return
        if key:
            name = Gtk.accelerator_name(key, mods)
            if name:
                fp.write("%s=%s\n" % (method, name))

    fp = open(filename, 'w')
    Gtk.AccelMap.foreach(write_shortcut_line, fp)
    fp.close()

def load_defaults():
    filename = dirs.get_first_config("shortcuts")
    if filename:
        load_main_from_file(filename)
    else:
        blank_all()

def save_defaults():
    filename = dirs.save_config_path("shortcuts")
    save_main_to_file(filename)

def _accel_changed_cb(object, path, key, mods):
    save_defaults()

