from . import streamtobuf


def buffer_line(text, line):
    line += text
    lines = line.split('\n')
    trailing_line = lines.pop()
    return lines, trailing_line


class StreamToBuffer(streamtobuf.StreamToBuffer):
    def __init__(self, buf, stop_callback = None, fds = None,
            disposable = True):
        self.incomplete_line = ""
        self.tb_extn = None
        streamtobuf.StreamToBuffer.__init__(self, buf, stop_callback,
                fds, disposable)


    def got_text(self, fd, text):
        completed_lines, self.incomplete_line = \
                buffer_line(text, self.incomplete_line)
        if not self.tb_extn:
            self.tb_extn = self.buffer.extns['throwback']
        for l in completed_lines:
            self.tb_extn.add_line(l)



class TermToBuffer(streamtobuf.TermToBuffer):
    " Derived classes may reimplement add_stdout_line and add_stderr_line "
    def __init__(self, buf, stop_callback = None,
            cmd_args = None, capture_stderr = True, cwd = None, shell = False,
            disposable = True):
        self.stdout_line = ""
        self.stderr_line = ""
        self.tb_extn = None
        streamtobuf.TermToBuffer.__init__(self, buf, stop_callback,
            cmd_args, capture_stderr, cwd, shell, disposable)


    def got_text(self, fd, text):
        if not self.tb_extn:
            self.tb_extn = self.buffer.extns['throwback']
        if fd == self.proc.stderr.fileno():
            completed_lines, self.stderr_line = \
                    buffer_line(text, self.stderr_line)
            adder = self.add_stderr_line
        else:
            completed_lines, self.stdout_line = \
                    buffer_line(text, self.stdout_line)
            adder = self.add_stdout_line
        if not self.tb_extn:
            self.tb_extn = self.buffer.extns['throwback']
        for l in completed_lines:
            adder(l)


    def add_stdout_line(self, line):
        self.tb_extn.add_line(line)


    def add_stderr_line(self, line):
        self.tb_extn.add_line(line)
