from gi.repository import Gdk
from gi.repository import Gtk
from gi.repository import GtkSource

from . import buffers
from . import settings
from . import vixiter


all_marks = []


class Mark(object):
    unique_id_index = 0
    """
    category: string "error", "warning", "info" or "find"
    tb_buf: buffer showing the throwback
    tb_start, tb_end: Iters marking where in tb_buf this tb is shown
    targ_filename: target filename
    targ_buf: target buffer
    plus_one: Offset to add to displayed lines and columns
    targ_line, targ_col: Position of throwback target in targ_buf
    targ_line_tb_start, targ_line_tb_end, targ_col_tb_start, targ_col_tb_end:
        Iters in tb_buf where target line and column numbers are shown
    """
    def __init__(self, category, tb_buf, tb_start, tb_end,
            targ_line, targ_line_tb_start, targ_line_tb_end,
            targ_filename, plus_one = 1, targ_buf = None,
            targ_col = 0, targ_col_tb_start = None, targ_col_tb_end = None,
            targ_text = ''):
        self.unique_id = Mark.unique_id_index
        Mark.unique_id_index += 1
        self.targ_text = targ_text
        self.tb_buf = tb_buf
        self.targ_filename = targ_filename
        self.targ_buf = targ_buf
        self.plus_one = plus_one
        self.orig_targ_line = targ_line
        self.orig_targ_col = targ_col
        self.reset_target_pos()
        self.tb_start_mark = tb_buf.create_mark(None, tb_start)
        self.tb_end_mark = tb_buf.create_mark(None, tb_end, True)
        self.tb_source_mark = tb_buf.create_source_mark(None,
                category, tb_start)
        if targ_line_tb_start:
            self.targ_line_tb_start_mark = tb_buf.create_mark(None,
                    targ_line_tb_start)
            self.targ_line_tb_end_mark = tb_buf.create_mark(None,
                    targ_line_tb_end, True)
        else:
            self.targ_line_tb_start_mark = None
            self.targ_line_tb_end_mark = None
        if targ_col_tb_start:
            self.targ_col_tb_start_mark = tb_buf.create_mark(None,
                    targ_col_tb_start)
            self.targ_col_tb_end_mark = tb_buf.create_mark(None,
                    targ_col_tb_end, True)
        else:
            self.targ_col_tb_start_mark = None
            self.targ_col_tb_end_mark = None
        self.targ_mark = None
        self.mark_set_tag = None
        if targ_buf:
            self.attach_to_buf(targ_buf)
        all_marks.append(self)


    def reset_target_pos(self):
        self.targ_line = self.orig_targ_line
        self.targ_col = self.orig_targ_col
        self.__update_tb_line_col(self.targ_line, self.targ_col)


    def confirm_target_pos(self):
        self.orig_targ_line = self.targ_line
        self.orig_targ_col = self.targ_col
        self.__update_tb_line_col(self.targ_line, self.targ_col)


    def attach_to_buf(self, targ_buf):
        " Note attach_to_buf adds self to targ_buf.throwbacks... "
        self.detach_from_buf()
        self.targ_buf = targ_buf
        iter = targ_buf.get_checked_iter(self.targ_col, self.targ_line)
        if not iter:
            iter = targ_buf.get_checked_iter(0, self.targ_line)
        if iter:
            self.targ_mark = targ_buf.create_mark(None, iter)
            self.mark_set_tag = targ_buf.connect("mark-set", self.__mark_set_cb)
            targ_buf.throwbacks.append(self)


    def detach_from_buf(self):
        " ...but detach_from_buf does not change targ_buf.throwbacks "
        if self.targ_buf:
            if self.targ_mark:
                self.targ_buf.delete_mark(self.targ_mark)
                self.targ_mark = None
            if self.mark_set_tag != None:
                self.targ_buf.disconnect(self.mark_set_tag)
                self.mark_set_tag = None
            self.targ_buf = None


    def __update_tb_line_col(self, line, col):
        if line != self.targ_line:
            self.targ_line = line
            if self.targ_line_tb_start:
                start = self.tb_buf.get_iter_at_mark(self.targ_line_tb_start)
                end = self.tb_buf.get_iter_at_mark(self.targ_line_tb_end)
                self.tb_buf.delete(start, end)
                start = self.tb_buf.get_iter_at_mark(self.targ_line_tb_start)
                self.tb_buf.insert(start, "%d" % (line + self.plus_one))
        if col != self.targ_col:
            self.targ_col = col
            if self.targ_col_tb_start:
                start = self.tb_buf.get_iter_at_mark(self.targ_col_tb_start)
                end = self.tb_buf.get_iter_at_mark(self.targ_col_tb_end)
                self.tb_buf.delete(start, end)
                start = self.tb_buf.get_iter_at_mark(self.targ_col_tb_start)
                self.tb_buf.insert(start, "%d" % (col + self.plus_one))


    def __mark_set_cb(self, targ_buf, iter, mark):
        if not self.targ_mark or mark != self.targ_mark:
            return
        self.__update_line_col(iter.get_line(), iter.get_line_offset())


    def destroy(self):
        self.tb_buf.delete_mark(self.tb_start_mark)
        self.tb_buf.delete_mark(self.tb_end_mark)
        self.tb_buf.delete_mark(self.tb_source_mark)
        if self.targ_line_tb_start_mark:
            self.tb_buf.delete_mark(self.targ_line_tb_start_mark)
        if self.targ_line_tb_end_mark:
            self.tb_buf.delete_mark(self.targ_line_tb_end_mark)
        if self.targ_col_tb_start_mark:
            self.tb_buf.delete_mark(self.targ_col_tb_start_mark)
        if self.targ_col_tb_end_mark:
            self.tb_buf.delete_mark(self.targ_col_tb_end_mark)
        if self.targ_buf:
            self.detach_from_buf()
        all_marks.remove(self)


    def activate(self, double_click = False):
        if self.targ_buf:
            v = self.targ_buf.get_view()
            w = v.get_vixwin()
            w.notebook.set_current_page(v.page_num)
            if self.targ_mark:
                iter = self.targ_buf.get_iter_at_mark(self.targ_mark)
                v.move_cursor_to_iter(iter)
            if double_click:
                w.present()
        elif double_click:
            # Can't import openfile globally because of circular dependencies
            from .openfile import open_file
            win = open_file(self.tb_buf.settings_client, None,
                    self.targ_filename)
            buf = win.get_buffer()
            self.attach_to_buf(buf)
            if self.targ_mark:
                iter = buf.get_iter_at_mark(self.targ_mark)
                win.view.move_cursor_to_iter(iter)



class BufExtn(object):
    """ Adds text a line at a time to the buffer, using virtual functions to
    check whether it should be made active. See above for plus_one. """
    def __init__(self, buf, plus_one = 1):
        self.buf = buf
        buf.extns['throwback'] = self
        self.plus_one = plus_one
        self.marks = []


    def add_line(self, line, *args, **kwargs):
        " args and kwargs are passed to parse_line virtual function "
        if line.endswith('\n'):
            line = line[:-1]
        b = self.buf
        start = b.get_end_iter()
        sm = b.create_mark(None, start, True)
        b.insert(start, line)
        end = b.get_end_iter()
        em = b.create_mark(None, end)
        b.insert(end, '\n')
        b.place_cursor(b.get_end_iter())
        start = b.get_iter_at_mark(sm)
        end = b.get_iter_at_mark(em)
        b.delete_mark(em)
        b.delete_mark(sm)
        filt = self.parse_line(line, *args, **kwargs)
        if filt:
            filename = filt[0]
            if self.plus_one:
                filt[1] -= self.plus_one
                if filt[2]:
                    filt[2] -= self.plus_one
            filename = self.buf.expandpath(filename)
            targ_buf = buffers.find_filename(filename)
            targ_line_tb_start = start.copy()
            if filt[3] != None:
                targ_line_tb_start = start.copy()
                targ_line_tb_start.set_line_offset(filt[3])
                targ_line_tb_end = start.copy()
                targ_line_tb_end.set_line_offset(filt[4])
            else:
                targ_line_tb_start = None
                targ_line_tb_end = None
            if filt[5] != None:
                targ_col_tb_start = start.copy()
                targ_col_tb_start.set_line_offset(filt[5])
                targ_col_tb_end = start.copy()
                targ_col_tb_end.set_line_offset(filt[6])
            else:
                targ_col_tb_start = None
                targ_col_tb_end = None
            mark = Mark(filt[7], b, start, end,
                    filt[1], targ_line_tb_start, targ_line_tb_end,
                    filename, self.plus_one, targ_buf,
                    filt[2], targ_col_tb_start, targ_col_tb_end, line)
            self.marks.append(mark)
            all_marks.append(mark)


    def destroy(self):
        for m in self.marks:
            m.destroy()


    # virtual def parse_line(self, line, *args, **kwargs):
        """
        If line matches a throwback pattern return:
        [targ_filename, targ_line, targ_col,
                line_start_offset, line_end_offset,
                col_start_offset, col_end_offset, category]
        offsets should be None if line/col numbers are unavailable
        """



class ViewExtn(object):
    def __init__(self, view):
        self.view = view
        view.extns['throwback'] = self
        self.buf_extn = None
        self.init_categories(view)


    def destroy(self):
        pass


    def click_event(self, event, iter):
        if event.button == 1 and (event.type == Gdk.EventType.BUTTON_PRESS or \
                event.type == Gdk.EventType._2BUTTON_PRESS):
            buf = self.view.get_buffer()
            if not self.buf_extn:
                self.buf_extn = buf.extns['throwback']
            for m in self.buf_extn.marks:
                c = iter.compare(buf.get_iter_at_mark(m.tb_start_mark))
                if c == 0 or (c > 0 and \
                        iter.compare(buf.get_iter_at_mark(m.tb_end_mark)) < 0):
                    m.activate(event.type == Gdk.EventType._2BUTTON_PRESS)
                    break
        return False


    @staticmethod
    def init_categories(view):
        attr = GtkSource.MarkAttributes()
        attr.set_stock_id(Gtk.STOCK_DIALOG_ERROR)
        view.set_mark_attributes("error", attr, 0)
        attr = GtkSource.MarkAttributes()
        attr.set_stock_id(Gtk.STOCK_DIALOG_WARNING)
        view.set_mark_attributes("warning", attr, 0)
        attr = GtkSource.MarkAttributes()
        attr.set_stock_id(Gtk.STOCK_DIALOG_INFO)
        view.set_mark_attributes("info", attr, 0)
        attr = GtkSource.MarkAttributes()
        attr.set_stock_id(Gtk.STOCK_FIND)
        view.set_mark_attributes("find", attr, 0)
        view.set_show_line_marks(True)
        view.set_highlight_current_line(True)



def add_view_extn_to_buf(buf, klass = ViewExtn):
    vext = None
    for v in buf.get_views():
        if vext:
            v.extns['throwback'] = vext
        else:
            vext = klass(v)



def register_buffer(buf):
    for t in all_marks:
        if t.targ_filename == buf.filename:
            t.attach_to_buf(buf)


# These are currently ignored
settings.register("throwwidth", int, None, None,
        _("Width of throwback windows in pixels (currently ignored)"))
settings.register("throwheight", int, None, None,
        _("Height of throwback windows in pixels (currently ignored)"))
