""" Some extra helper functions for iters etc. """


def move_iter_by_paras(iter, count):
    while count != 0:
        if count > 0:
            while True:
                if not iter.forward_line():
                    iter.forward_to_line_end()
                    return
                iter.set_visible_line_offset(0)
                if iter.ends_line():
                    count -= 1
                    break
        else:
            while True:
                iter.set_visible_line_offset(0)
                if not iter.backward_line():
                    return
                if iter.ends_line():
                    count += 1
                    break
        

def iter_to_first_non_space(iter):
    iter.set_visible_line_offset(0)
    while not iter.ends_line() and iter.get_char().isspace():
        iter.forward_cursor_position()


def extend_iters_to_whole_lines(iter1, iter2):
    if iter2.compare(iter1) > 0:
        end_iter = iter2
        start_iter = iter1
    else:
        end_iter = iter1
        start_iter = iter2
    if not start_iter.starts_line():
        start_iter.set_line_offset(0)
    if not end_iter.starts_line():
        if not end_iter.ends_line():
            end_iter.forward_line()
        end_iter.set_line_offset(0)


def iter_to_eol(iter, exclude_nl = False):
    """ Moves an iter to the start of the next line, or to the end of the
    current line if there are no more lines or exclude_nl is True. """
    if exclude_nl:
        if not iter.ends_line():
            iter.forward_to_line_end()
        return
    l = iter.get_line()
    iter.forward_line()
    if iter.get_line() == l:
        iter.forward_to_line_end()
    else:
        iter.set_line_offset(0)


def ensure_iter_eol(iter, exclude_nl = False):
    """ Calls iter_to_eol iff iter isn't already at the start/end of a line """
    if exclude_nl or iter.get_line_offset() != 0:
        iter_to_eol(iter, exclude_nl)


def get_iter_pos(iter):
    return (iter.get_line_offset(), iter.get_line())


def describe(iter):
    return "(%s, %s)" % get_iter_pos(iter)


def get_column(iter, ts):
    """ Calculates an iter's visible column taking tabs into account. """
    iter1 = iter.copy()
    iter1.set_visible_line_offset(0)
    col = 0
    while not iter1.equal(iter):
        if iter1.get_char() == '\t':
            col += ts - col % ts
        else:
            col += 1
        iter1.forward_cursor_position()
    return col
